/****
 *
 * Ttulo: LeituraFacil
 *
 * Autor: Ulysses de Oliveira
 *
 * Data de Criao: 15/07/2012
 * ltima modificao: 10/10/2013
 *
 * Descrio: Implementao da biblioteca LeituraFacil
 *
 ****/

/************************ Includes ************************/

#include <stdio.h>   /* Entrada e sada             */
#include <string.h>  /* Processamento de strings    */
#include <ctype.h>   /* Classificao de caracteres */
#include "leitura.h" /* Interface da biblioteca */
                     /* LeituraFacil            */

/******************** Funes de Locais *******************/

/****
 *
 * LimpaBuffer(): Limpa o buffer associado ao meio de
 *                entrada padro (stdin)
 *
 * Parmetros: Nenhum
 *
 * Retorno: O nmero de caracteres descartados, com
 *          exceo do caractere '\n'.
 *
 ****/
static int LimpaBuffer(void)
{
   int carLido, /* Armazena cada caractere lido */
       nCarLidos = 0; /* Conta o nmero de */
                      /* caracteres lidos  */

      /* L e descarta cada caractere lido at    */
      /* encontrar '\n' ou getchar() retornar EOF */
   do {
      carLido = getchar(); /* L um caractere */
      ++nCarLidos; /* Mais um caractere foi lido */
   } while ((carLido != '\n') && (carLido != EOF));

      /* O ltimo caractere lido foi '\n' ou  */
      /* EOF e no deve ser considerado sobra */
   return nCarLidos - 1;
}

/****
 *
 * MostraMensagem(): Apresenta uma mensagem na tela
 *                   com embelezamento
 *
 * Parmetros: msg (entrada) - mensagem a ser apresentada
 *
 * Retorno: Nada
 *
 * Observao: O objetivo desta funo  uniformizar as
 *             mensagens emitidas pelas funes globais
 *             da biblioteca LeituraFacil
 *
 ****/
static void MostraMensagem(const char *msg)
{
   printf("\a\n\t>>> %s", msg);
}

/****
 *
 * InformaDescarte(): Apresenta uma mensagem na tela
 *                    informando quantos caracteres
 *                    foram descartados, se for o caso
 *
 * Parmetros: n (entrada) - nmero de caracteres
 *                           descartados
 *
 * Retorno: Nada
 *
 * Observao: O objetivo desta funo  uniformizar as
 *             mensagens emitidas pelas funes globais
 *             da biblioteca LeituraFacil que informam
 *             que o usurio digitou mais do que deveria
 *
 ****/
static void InformaDescarte(int n)
{
      /* Verifica se h mensagem a ser apresentada */
   if (n <= 0) {
      return; /* No h */
   }

      /* Apresenta a mensagem adequada para a situao */
   if (n == 1) {
      printf("\t>>> Um caractere foi descartado\n");
   } else {
      printf("\t>>> %d caracteres foram descartados\n", n);
   }
}

/******************** Funes de Globais ******************/

/****
 *
 * LeCaractere(): L um caractere via teclado e remove os
 *                caracteres remanescentes no buffer
 *
 * Parmetros: Nenhum
 *
 * Retorno: O caractere lido
 *
 * Observao: Esta funo deixa o buffer limpo
 *
 ****/
int LeCaractere(void)
{
   int c, /* Armazenar cada caractere lido */
       nResto = 0; /* Nmero de caracteres */
                   /* excedentes           */

   /* Volta para c se ocorrer erro de leitura */
inicio:
   c = getchar(); /* L o caractere digitado */

      /* Verifica se getchar() retornou EOF */
   if (c == EOF) {
      MostraMensagem( "Erro de leitura. "
                      "Tente novamente\n\t> " );
      goto inicio; /* No faz mal algum */
   }

      /* Pelo menos '\n' se encontra no buffer */
   nResto = LimpaBuffer();

      /* Repreende o usurio se ele digitou demais */
   InformaDescarte(nResto);

   return c; /* Retorna o caractere lido */
}

/****
 *
 * LeInteiro(): L um nmero inteiro via teclado
 *              e deixa o buffer intacto
 *
 * Parmetros: Nenhum
 *
 * Retorno: O nmero inteiro lido
 *
 ****/
int LeInteiro(void)
{
   int num, /* O nmero lido */
       teste, /* Valor retornado por scanf() */
       nResto = 0; /* Nmero de caracteres */
                   /* excedentes           */

   /* Desvia para c se o valor for invlido */
inicio:
      /* Tenta ler um valor vlido */
   teste = scanf("%d", &num);

      /* Se no ocorreu erro de leitura ou de final */
      /* de arquivo, h caracteres remanescentes    */
      /* que precisam ser removidos                 */
   if (teste != EOF) {
      nResto = LimpaBuffer();
   }

      /* Enquanto o valor retornado por scanf()  */
      /* no indicar que um valor vlido foi     */
      /* lido continua tentando obter esse valor */
   while(teste != 1) {
      MostraMensagem( "O valor digitado e' invalido. "
                      "Tente novamente\n\t> " );
      goto inicio; /* No causa dano algum */
   }

      /* Se for o caso, repreende o usurio por */
      /* ele ter digitado mais do que deveria   */
   InformaDescarte(nResto);

      /* O valor retornado certamente  vlido */
   return num;
}

/****
 *
 * LeReal(): L um valor do tipo double via teclado
 *           deixando o buffer associado ao teclado
 *           sem caracteres remanescentes
 *
 * Parmetros: Nenhum
 *
 * Retorno: O valor lido e validado
 *
 ****/
double LeReal(void)
{
   int    teste, /* Armazena o retorno de scanf() */
          nResto = 0; /* Nmero de caracteres */
                      /* excedentes           */
   double valorLido; /* O valor digitado pelo usurio */

      /* O lao encerra quando o usurio se comporta bem */
   while (1) {
         /* Tenta ler um valor do tipo double */
      teste = scanf("%lf", &valorLido);

         /* Se no ocorreu erro de leitura ou de final */
         /* de arquivo, h caracteres remanescentes    */
         /* que precisam ser removidos                 */
      if (teste != EOF) {
         nResto = LimpaBuffer();
      }

         /* Se o valor retornado por scanf() */
         /* foi 1, a leitura foi OK          */
      if (teste == 1) {
         break; /* Leitura foi nota 10 */
      } else { /* Usurio foi mal comportado */
         MostraMensagem( "O valor digitado e' invalido. "
                         "Tente novamente\n\t> " );
      }
   }

      /* Se for o caso, repreende o usurio */
   InformaDescarte(nResto);

      /* Retorna um valor vlido do tipo double */
   return valorLido;
}

/****
 *
 * LeString(): L um string via teclado deixando o buffer
 *             zerado
 *
 * Parmetros:
 *      ar[] (sada): array que armazenar o string lido
 *      nElementos (entrada): nmero mximo de elementos que
 *                            sero armazenado no array,
 *                            incluindo '\0'
 *
 * Retorno: O nmero de caracteres que o usurio digitou a
 *          mais, sem incluir '\n'
 *
 * Observaes:
 *      1. "Ler um string" significa ler caracteres,
 *         armazen-los num array e acrescentar o
 *         caractere '\0' ao final para que o array
 *         contenha um string.
 *      2. O nmero mximo de caracteres lidos ser
 *         nElementos - 1, porque se deve deixar um
 *         espao sobressalente para o caractere '\0'.
 *      3. Se o caractere '\n' for lido, ele no far
 *         parte do string resultante.
 *
 ****/
int LeString(char *ar, int nElementos)
{
   char *p; /* Usado para remover '\n' do string */
   int   nCarDeixados = 0; /* Conta o nmero de     */
                           /* caracteres excedentes */
                           /* no buffer             */

      /* L no mximo nElementos - 1 caracteres ou at    */
      /* encontrar '\n'. Quando fgets() retorna NULL,     */
      /* ocorreu erro ou tentativa de leitura alm do     */
      /* final do arquivo. Nesses casos, o lao continua. */
   while ( fgets(ar, nElementos, stdin) == NULL )  {
      printf( "\a\n\t>>> %s", "Erro de leitura. "
              "Tente novamente\n\t> " );
   }

      /* Faz p apontar para o caractere que      */
      /* antecede o caractere terminal do string */
   p = strchr(ar, '\0') - 1;

      /* Se o caractere '\n' foi lido, remove-o do   */
      /* string. Caso contrrio, remove-o do buffer. */
   if (*p == '\n') {
         /* Caractere '\n' faz parte do string */
      *p = '\0'; /* Sobrescreve '\n' com '\0' */
   } else { /* Usurio digitou caracteres demais */
      nCarDeixados = LimpaBuffer();
   }

      /* Retorna o nmero de caracteres que o usurio */
      /* digitou alm do esperado, sem incluir '\n'   */
   return nCarDeixados;
}

/****
 *
 * LeOpcao(): L, via teclado, um caractere que deve fazer
 *            parte de um string para ser considerado vlido
 *
 * Parmetros:
 *      opcoes (entrada) - string contendo os caracteres
 *                         vlidos
 *
 * Retorno: Um caractere vlido
 *
 * Observaes:
 *      1. Esta funo deixa o buffer virgem novamente
 *      2. Esta funo  denominada 'LeOpcao' porque o
 *         caractere lido e validado  restrito a um
 *         conjunto de caracteres (opes) representado
 *         pelo string recebido como parmetro
 *
 ****/
int LeOpcao(const char *opcoes)
{
   int op; /* Opo (caractere) escolhida pelo usurio */

      /* Enquanto o usurio no escolher uma opo */
      /* vlida, o lao a seguir no encerra       */
   while (1) {
         /* L o caractere digitado */
      op = LeCaractere();

         /* Verifica se o caractere  vlido */
      if (strchr(opcoes, op)) {
         break; /*  */
      } else { /* No  */
         MostraMensagem( "Opcao incorreta. "
                         "Tente novamente\n\t> " );
      }
   }

      /* Certamente, a opo retornada  vlida */
   return op;
}

